#!/usr/bin/env python

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import os.path
import exceptions
import tempfile
import urlgrabber
import logger
import sysutils
from config import *

class BootLoaderError (exceptions.Exception):
    def __init__ (self, args = None):
        exceptions.Exception.__init__ (self)
        self.args = args
        
def _download_kernel (repository, kernelversion, destdir):
    kernel = "vmlinuz-" + kernelversion
    url = repository + "/" + kernel

    logger.verbose ("Downloading kernel '%s' to '%s'" % (url, destdir))

    try:
        urlgrabber.urlgrab (url,
                            os.path.join (destdir, kernel),
                            copy_local = 1)
    except urlgrabber.grabber.URLGrabError, e:
        raise BootLoaderError (_("Failed to download '%s' : %s") % (url, e))

    return kernel

#
# FIXME: obviously this is x86 specific. It'd be nice to use
#        booty, but the anaconda dependancies there make that
#        difficult
#
# FIXME: no bootsplash
#
# FIXME: no xen support
#
def _install_bootloader (title, device, rootdev, chroot, kernel, initrd):
    bootdir = os.path.join (chroot, "boot")
    grubdir = os.path.join (bootdir, "grub")
    grubconf = os.path.join (grubdir, "grub.conf")

    bios_num = sysutils.get_bios_number (device)

    # Forcibly disable selinux until readonly-root works with it
    extraargs = "selinux=0"
        
    os.mkdir (grubdir)

    logger.verbose ("Writing %s" % grubconf)

    f = file (grubconf, "w")
    f.write ("title %s\n" % title)
    f.write ("\troot (hd%d,0)\n" % bios_num)
    f.write ("\tkernel /%s ro root=%s %s\n" % (kernel, rootdev, extraargs))
    f.write ("\tinitrd /%s\n" % initrd)
    f.close ()

    os.chmod (grubconf, 0600)
    os.symlink ("grub.conf", os.path.join (grubdir, "menu.lst"))

    devmap_path = os.path.join (grubdir, "device.map")

    f = file (devmap_path, "w")
    f.write ("(hd%d) %s\n" % (bios_num, device))
    f.close ()

    logger.run_command (GRUB_INSTALL_PATH + " --root-directory=" + chroot +
                        " --just-copy " + device)

    (fd, grubscript) = tempfile.mkstemp (dir = "/tmp", prefix = "grubscript-")
    f = os.fdopen (fd, "w")
    f.write ("root (hd%d,0)\n" % bios_num)
    f.write ("install /grub/stage1 d " +
             "(hd%d) /grub/stage2 p /grub/grub.conf\n" % bios_num)
    f.close ()
        
    try:
        #
        # FIXME: the allow_execmem thing is a temporary hack until
        #        https://bugzilla.redhat.com/194452 is fixed
        #
        logger.run_command ("/usr/sbin/setsebool allow_execmem 1", True)
        logger.run_command (GRUB_PATH + " --batch --no-floppy " +
                            "--device-map=" + devmap_path +
                            " < " + grubscript)
        logger.run_command ("/usr/sbin/setsebool allow_execmem 0", True)
    finally:
        os.remove (grubscript)

def _mkinitrd (chroot, kernelversion, lvmdev):
    initrd = "initrd-" + kernelversion + ".img"
        
    def tmpmount (chroot, subdir, type):
        tmpdir = os.path.join (chroot, subdir)
        logger.run_command (MOUNT_PATH + " -t " + type + " none " + tmpdir)
        return tmpdir

    # Mount various bits inside the chroot
    tmpmounts = []
    tmpmounts.append (tmpmount (chroot, "tmp", "tmpfs"))
    tmpmounts.append (tmpmount (chroot, "dev", "tmpfs"))
    tmpmounts.append (tmpmount (chroot, "var/lock", "tmpfs"))
    tmpmounts.append (tmpmount (chroot, "proc", "proc"))
    tmpmounts.append (tmpmount (chroot, "sys", "sysfs"))

    # Need to be able to activate the VG inside the chroot
    st = os.stat (lvmdev)
    os.mknod (os.path.join (chroot, lvmdev[1:]), st.st_mode, st.st_rdev)

    try:
        logger.verbose ("Creating initrd '%s'" % initrd)

        pid = os.fork ()
        if pid == 0: # child
            os.chroot (chroot)
            logger.run_command (MKINITRD_PATH + " -f /boot/" +
                                initrd + " " + kernelversion)
            os._exit (0)

        os.waitpid (pid, 0)
    finally:
        for m in tmpmounts:
            logger.run_command (UMOUNT_PATH + " " + m)

    return initrd

def update_bootloader (image, device, bootdev, vgi, repository, kernelversion):
    logger.verbose ("Updating bootloader with kernel '%s'" % kernelversion)
    
    chroot = tempfile.mkdtemp (dir = "/tmp", prefix = "root-")
    bootdir = os.path.join (chroot, "boot")

    logger.run_command (MOUNT_PATH + " -o ro " + vgi.rootlv_path + " " + chroot)
    logger.run_command (MOUNT_PATH + " " + bootdev + " " + bootdir)
        
    try:
        kernel = _download_kernel (repository, kernelversion, bootdir)

        initrd = _mkinitrd (chroot, kernelversion, vgi.lvmdev)

        _install_bootloader (image.name, device, vgi.rootlv_path, chroot, kernel, initrd)
        
        # FIXME: delete old kernels
    finally:
        logger.run_command (UMOUNT_PATH + " " + bootdir)
        logger.run_command (UMOUNT_PATH + " " + chroot)
        os.rmdir (chroot)
