#!/usr/bin/env python

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import os.path
import exceptions
import urlgrabber
import libxml2
import fileutils
import logger

#
# FIXME: add the following to the metadata:
#   + Updates information
#   + VG extent size
#   + Snapshot chunk size
#

class ImageError (exceptions.Exception):
    def __init__ (self, args = None):
        exceptions.Exception.__init__ (self)
        self.args = args

class Image:
    """Encapsulates the metadata of a single image. The following
    member attributes are available:

     - @name: the name of the image
     - @size: the size, in bytes, of the image file
     - @kernelversion: the version of the kernel to be used .. the
                       repository contains the file vmlinuz-@kernelversion
     - @volumegroup: the LVM volume group in which @rootlv and @swaplv
                     should be created corresponding to the names in e.g.
                     /etc/fstab
     - @rootlv: the name of the volume which will contain the root filessystem
     - @swaplv: the name of the volume which will be used for swap space.

    The @revisions member is a list of (@id, @size, @kernelversion) tuple
    describing each of the revisions available for that image.
    """

    def __init__ (self, node):
        """Initialize an %Image object by parsing the contents
        of the XML @node.
        """
        self.revisions = []
        
        self._parse (node)

    def _get_value (self, parent, child_name):
        for child in parent.children:
            if child.name == child_name:
                return child.content
        raise ImageError (_("Invalid image list data: required <%s> element not found") %
                          child_name)
    
    def _parse_revision (self, revision_node):
        id = self._get_value (revision_node, "id")
        size = self._get_value (revision_node, "size")
        kernelversion = self._get_value (revision_node, "kernelversion")

        try:
            id = int (id)
        except ValueError:
            raise ImageError (_("Invalid revision data: <id> value is not an integer"))

        try:
            size = long (size)
        except ValueError:
            raise ImageError (_("Invalid revision data: <size> value is not an integer"))

        self.revisions.append ((id, size, kernelversion))
        
    def _parse (self, image_node):
        self.name          = self._get_value (image_node, "name")
        self.size          = self._get_value (image_node, "size")
        self.kernelversion = self._get_value (image_node, "kernelversion")
        self.volumegroup   = self._get_value (image_node, "volumegroup")
        self.rootlv        = self._get_value (image_node, "rootlv")
        self.swaplv        = self._get_value (image_node, "swaplv")

        try:
            self.size = long (self.size)
        except ValueError:
            raise ImageError (_("Invalid image list data: <size> value is not an integer"))

        for node in image_node.children:
            if node.name == "revision":
                self._parse_revision (node)

        self.revisions.sort (lambda a, b: cmp (a[0], b[0]))

class ImageList:
    """Encapsulates a list of %Image objects. The list
    is available in the @images member.
    """
    
    def __init__ (self, metadata):
        """Initialize an %ImageList object by parsing the metadata XML
        supplied in @metadata.
        
        The format is:
        
          <images>
            <image>
              <name>foo</name>
              <size></size>
              <kernelversion></kernelversion>
              <volumegroup></volumegroup>
              <rootlv></rootlv>
              <swaplv></swaplv>

              <revision>
                <id>1</id>
                <size></size>
                <kernelversion></kernelversion>
              </revision>
                
              <revision>
                <id>2</id>
                <size></size>
                <kernelversion></kernelversion>
              </revision>
                
            </image>
          </images>
          
        Note, the size is specified in bytes.
        """
        self.images = []

        if metadata:
            self._parse (metadata)

    def _parse (self, data):
        try:
            doc = libxml2.readMemory (data, len (data), None, None,
                                      libxml2.XML_PARSE_NOBLANKS)
        except libxml2.libxmlError, e:
            raise ImageError (_("Failed to parse image list data: %s") % e)

        root = doc.getRootElement ()
        if not root or root.name != "images":
            raise ImageError (_("Invalid image list data: root element is not 'images'"))

        for node in root.children:
            if node.name == "image":
                self.images.append (Image (node))

def get_image (repo, image_name, cache_dir = None):
    """Fetch metadata for @image_name from the image
    repository at @repo and return a %Image object
    encapsulating the metadata.

    If @cache_dir is supplied, it should be a writable
    directory where the imagemetadata.xml file can
    be cached. If the file is already cached and is no
    older than the file in the repository, #None is
    returned to indicate that no change has occurred.

    Raises an ImageError exception if the metadata
    cannot be found.
    """
    metadata_url = repo + "/imagemetadata.xml"
    
    logger.verbose ("Grabbing '%s'" % metadata_url)
    
    try:
        if cache_dir is None:
            metadata = urlgrabber.urlread (metadata_url)
        else:
            cache_file = os.path.join (cache_dir, "imagemetadata.xml")

            mtime = 0
            if os.path.exists (cache_file):
                mtime = os.stat (cache_file).st_mtime
            
            urlgrabber.urlgrab (metadata_url,
                                cache_file,
                                copy_local = 1,
                                reget = 'check_timestamp')

            if os.stat (cache_file).st_mtime <= mtime:
                logger.verbose ("Cached file '%s' is no older than '%s'" %
                                (cache_file, metadata_url))
                return None

            metadata = fileutils.read_file (cache_file)
            
    except urlgrabber.grabber.URLGrabError, e:
        raise ImageError (_("Failed to read '%s/imagemetadata.xml': %s") % (repo, e))

    for image in ImageList (metadata).images:
        if image.name == image_name:
            return image

    raise ImageError (_("Image '%s' not found in '%s/imagemetadata.xml'") % (image_name, repo))

def clear_cache (cache_dir):
    """Remove any cached imagemetadata.xml file from @cache_dir."""
    cache_file = os.path.join (cache_dir, "imagemetadata.xml")
    if os.path.exists (cache_file):
        os.remove (cache_file)
