#!/usr/bin/python

#
#  Copyright (C) 2006 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import logger
from config import *

class Config:
    def __init__ (self, path = CONFIG_PATH):
        self._path = path
        self._shvar = shvarFile (path, False)
        
        self.repository_url = self._shvar["REPOSITORY_URL"]
        self.image_name     = self._shvar["IMAGE_NAME"]
        self.image_revision = self._shvar["IMAGE_REVISION"]

        logger.verbose ("Read config from '%s': repo URL '%s', image '%s', revision '%s'" %
                        (self._path, self.repository_url, self.image_name, self.image_revision))


#
# Handle files with only shell variable declarations and full-line comments
# 
# This is based on the shvar.c from Red Hat's C configuration tools, and
# has similar functions, but differs in various aspects:
#
#  - It doesn't do inheritance
#  - Empty variables aren't treated the same as missing variables
#  - The shell unquoting is closer to correct
#

import re

def unescape_double (str):
    return re.sub ("\\\\([$`\"\\\\])", "\\1", str)

def unescape_unquoted(str):
    return re.sub ("\\\\(.)", "\\1", str)

def unescape_partial (match):
    s = match.group ()
    if s[0] == '"':
        return unescape_double (s[1:-1])
    elif s[0] == "'":
        return s[1:-1]
    else:
        return unescape_unquoted (s)

# This is not exactly right; one thing that is definitely wrong is
# the handling of newlines (but the splitting into lines doesn't handle
# that correctly,anyways). Also, we don't guard against unmatched
# quotes; they'll just get passed through verbatim
def unescape(val):
    return re.sub ("[^'\"]+|'[^']*'|\"([^\\\\\\\"]+|\\\\.)*\"", unescape_partial, val)

class shvarFile:
    def __init__ (self, filename, mustExist = False):
        self._lines = []

        try:
            f = file (filename, "r")
        except IOError:
            if mustExist:
                raise
            else:
                return

        while True:
            l = f.readline ()
            if l == "":
                break
            self._lines.append (l.rstrip())
            
        f.close ()

    # Get the value associated with the key. An empty value
    # is treated the same as a missing value; in both cases None is returned
    def __getitem__ (self, key):
        prefix = key + "="
        for l in self._lines:
            if l.startswith (prefix):
                value = unescape (l[len(prefix):])
                if value:
                    return value
        return None

    # return 1 if <key> resolves to any truth value (e.g. "yes", "y", "true")
    # return 0 if <key> resolves to any non-truth value (e.g. "no", "n", "false")
    # return <default> otherwise
    def bool (self, key, default = False):
        s = self[key]
        if not s:
            return default
        s = s.lower ()
        if s == "yes" or s == "true" or s == "y" or s == "t":
            return True
        elif s == "no" or s == "false" or s == "f" or s == "n":
            return 0
        else:
            return default

