#!/usr/bin/env python

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#
# Connect to /org/kernel/udev/monitor and receive events.
# Of course, this doesn't work if udevmonitor is already
# running so, in that case, we have to fall back to a timeout
#

import errno
import sys
import socket
import select
import time
import logger
from config import *

class UDevMonitor:
    def __init__ (self):
        self.sock = None
        self.open ()

    def __del__ (self):
        self.close ()

    def open (self):
        self.sock = socket.socket (socket.AF_UNIX, socket.SOCK_DGRAM)
        try:
            self.sock.bind ("\x00" + UDEV_MONITOR_PATH)
        except socket.error, (err, str):
            self.close ()
            if err != errno.EADDRINUSE:
                raise
            sys.stderr.write (_("Failed to monitor for udev events: "
                                "udevmonitor already running?\n"))

    def close (self):
        if self.sock:
            self.sock.close ()
            self.sock = None

    def wait_for_event (self, action, devpath, timeout = None):
        if not self.sock:
            logger.verbose ("Sleeping for %f seconds" % timeout)
            time.sleep (timeout)
            return
        
        start = time.time ()
        wait_for = action + "@" + devpath
        
        logger.verbose ("Waiting for udev event '%s'" % wait_for)
        
        while True:
            ready = []
            try:
                if timeout is None:
                    ready = select.select ([self.sock], [], [])[0]
                else:
                    ready = select.select ([self.sock], [], [], timeout)[0]
            except select.error, (err, str):
                if err != errno.EINTR:
                    raise
                logger.verbose ("Select interrupted by a signal")

            if not ready:
                if timeout is None:
                    continue
                else:
                    current = time.time ()
                    if current >= start + timeout:
                        logger.verbose ("Timeout expired")
                        break
                    else:
                        logger.verbose ("Resetting timeout to %f from %f" %
                                        (timeout - (current - start), timeout))
                        timeout -= current - start
                        continue
            
            buf = self.sock.recv (2048)
            if buf and buf.find ("\x00") > 0:
                event = buf[:buf.index("\x00")]
                logger.verbose ("Received udev event '%s'" % event)
                if event == wait_for:
                    return True

        return False
