#!/usr/bin/env python

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import exceptions
import parted
from config import *

from stacaccli.common import bootloader
from stacaccli.common import grabutils
from stacaccli.common import imagemetadata
from stacaccli.common import logger
from stacaccli.common import lvm
from stacaccli.common import sysutils
from stacaccli.common import udevmonitor
from stacaccli.common.config import *

class InstallerError (exceptions.Exception):
    def __init__ (self, args = None):
        exceptions.Exception.__init__ (self)
        self.args = args

class Install:
    def __init__ (self, repo, image_name, device, force = False):
        self._repo = repo
        self._image_name = image_name
        self._device = device
        self._force = force

        self._image = None

        self._boot_part = None
        self._lvm_part = None

        self._vgi = None

    def _prompt (self, message):
        if self._force:
            return True

        while True:
            c = raw_input ("%s [y/N]: " % message).lower ()
            if not c or c[0] == "n":
                return False
            elif c[0] == "y":
                return True

    def _verbose_dump_parts (self, parts):
        fmt = "%-6s  %-10s  %-6s  %-6s  %-12s  %-12s  %-12s  %-6s"
        
        logger.verbose (fmt % ("Label", "Path", "Type", "FS", "Start", "End", "Length (Mb)", "Flags"))
        
        for (label, part) in parts:
            if part.fs_type:
                fs_type_name = part.fs_type.name
            else:
                fs_type_name = "(none)"

            flags = ""
            if part.is_active ():
                flag = parted.partition_flag_next (0)
                while flag:
                    if part.get_flag (flag):
                        flags += parted.partition_flag_get_name (flag) + " "
                    flag = parted.partition_flag_next (flag)
            if not flags:
                flags = "(none)"

            logger.verbose (fmt % (label, part.geom.dev.path + str (part.num),
                                   part.type, fs_type_name, part.geom.start,
                                   part.geom.end,
                                   "%.2f" % (part.geom.length * part.geom.dev.sector_size / 1024.0 / 1024.0),
                                   flags))
        
    def _create_partitions (self):
        dev = parted.PedDevice.get (self._device)
        
        disk = dev.disk_new_fresh (parted.disk_type_get ("msdos"))

        any_constraint = dev.constraint_any ()

        logger.verbose ("Creating /boot partition")
        start = 0
        end = (BOOT_PART_SIZE * 1024L * 1024L) / dev.sector_size
        self._boot_part = disk.partition_new (parted.PARTITION_PRIMARY,
                                              None, start, end)

        disk.add_partition (self._boot_part, any_constraint)

        logger.verbose ("Creating LVM volume group partition")
        start = self._boot_part.geom.end
        end = dev.length - 1
        self._lvm_part = disk.partition_new (parted.PARTITION_PRIMARY,
                                             None, start, end)
        
        disk.add_partition (self._lvm_part, any_constraint)
        disk.maximize_partition (self._lvm_part, any_constraint)

        logger.verbose ("Committing partition table to disk:")
        self._verbose_dump_parts ([("/boot", self._boot_part),
                                   ("LVM VG", self._lvm_part)])
        
        disk.commit ()

    def _format_boot_part (self):
        bootdev = sysutils.get_partition_path (self._boot_part)

        logger.verbose ("Formatting and labelling /boot partition '%s'" %
                        bootdev)
        
        logger.run_command (MKE2FS_PATH + " -j " + bootdev)
        logger.run_command (TUNE2FS_PATH + " -c0 -i0 -Odir_index " + bootdev)
        logger.run_command (E2LABEL_PATH + " " + bootdev + " /boot")

    def _setup_lvm_part (self):
        self._vgi = lvm.VolumeGroupInfo (sysutils.get_partition_path (self._lvm_part),
                                         self._image.volumegroup,
                                         self._image.rootlv,
                                         self._image.swaplv)

        lvm.pvcreate (self._vgi.lvmdev)
        lvm.vgcreate (self._vgi.volumegroup, self._vgi.lvmdev)

        # FIXME: need to handle not having enough space.
        #        image.size isn't enough to actually hold an entirely
        #        net image ... don't forget about the metadata areas
        #        Should the snapshot size be in the metadata?
        
        lvm.lvcreate (self._vgi.volumegroup,
                      self._vgi.originlv,
                      self._image.size)
        lvm.lvcreate (self._vgi.volumegroup,
                      self._vgi.updatelv,
                      self._image.size)
        lvm.lvcreate (self._vgi.volumegroup,
                      self._vgi.swaplv,
                      sysutils.get_swap_size ())

    def _setup_rootlv (self):
        lvm.lvcreate (self._vgi.volumegroup,
                      self._vgi.rootlv,
                      self._image.size,
                      True,
                      self._vgi.originlv_path)

    #
    # FIXME: how can we sure we'll actually swapon this device?
    #        swapon or some other way to make sure it's in
    #        /etc/fstab ?
    #
    def _format_swaplv (self):
        logger.verbose ("Setting up swap device '%s'" % self._vgi.swaplv_path)
        logger.run_command (MKSWAP_PATH + " " + self._vgi.swaplv_path)

    #
    # FIXME: this is a really sucky way of retrieving the image ...
    #        e.g. if this is a 20Gb image with only 2Gb of data on
    #        it, then we'll still download the while 20Gb
    #
    def _copy_image_to_originlv (self):
        grabutils.urlgrab_to_dev (self._repo + "/" + self._image.name + ".img",
                                  self._image.size,
                                  self._vgi.originlv_path)

    def run (self):
        try:
            self._image = imagemetadata.get_image (self._repo, self._image_name)
        except imagemetadata.ImageError, e:
            raise InstallerError (_("Failed to install image '%s' : %s") %
                                  (self._image_name, e))

        logger.verbose ("Installing image '%s' to device '%s'" %
                        (self._image_name, self._device))

        if not self._prompt (_("Overwrite the contents of device '%s'?") %
                             self._device):
            logger.verbose ("Exiting ...")
            return

        lvm.deactivate_vgs_on_dev (self._device)

        umon = udevmonitor.UDevMonitor ()

        self._create_partitions ()

        umon.wait_for_event ("add",
                             sysutils.get_partition_udev_path (self._boot_part),
                             UDEV_WAIT_TIMEOUT)
        umon.close ()
        
        self._format_boot_part ()
        self._setup_lvm_part ()
        
        try:
            self._format_swaplv ()
            self._copy_image_to_originlv ()
            self._setup_rootlv ()

            bootloader.update_bootloader (self._image,
                                          self._device,
                                          sysutils.get_partition_path (self._boot_part),
                                          self._vgi,
                                          self._repo,
                                          self._image.kernelversion)
        finally:
            lvm.vgchange (self._image.volumegroup, False)
